@@ -8,15 +8,15 @@ use clvm_rs::allocator::{Allocator, NodePtr, SExp};
 use clvm_rs::reduction::EvalErr;
 
 use crate::classic::clvm::__type_compatibility__::{Bytes, BytesFromType, Record, Stream};
-use crate::classic::clvm::{KEYWORD_FROM_ATOM, KEYWORD_TO_ATOM};
+use crate::classic::clvm::{keyword_from_atom, keyword_to_atom};
+use crate::classic::clvm_tools::ir::r#type::IRRepr;
 use crate::classic::clvm_tools::ir::reader::IRReader;
 use crate::classic::clvm_tools::ir::writer::write_ir;
-use crate::classic::clvm_tools::ir::Type::IRRepr;
 
-pub fn is_printable_string(s: &String) -> bool {
+pub fn is_printable_string(s: &str) -> bool {
     for ch in s.graphemes(true) {
-        if ch.chars().nth(0).unwrap() > 0xff as char
-            || !is_printable(ch.chars().nth(0).unwrap() as u8)
+        if ch.chars().next().unwrap() > 0xff as char
+            || !is_printable(ch.chars().next().unwrap() as u8)
         {
             return false;
         }
@@ -24,8 +24,8 @@ pub fn is_printable_string(s: &String) -> bool {
     true
 }
 
-pub fn assemble_from_ir<'a>(
-    allocator: &'a mut Allocator,
+pub fn assemble_from_ir(
+    allocator: &mut Allocator,
     ir_sexp: Rc<IRRepr>,
 ) -> Result<NodePtr, EvalErr> {
     match ir_sexp.borrow() {
@@ -35,11 +35,11 @@ pub fn assemble_from_ir<'a>(
         IRRepr::Hex(b) => allocator.new_atom(b.data()),
         IRRepr::Symbol(s) => {
             let mut s_real_name = s.clone();
-            if s.starts_with("#") {
-                s_real_name = s[1..].to_string();
+            if let Some(stripped) = s.strip_prefix('#') {
+                s_real_name = stripped.to_string();
             }
 
-            match KEYWORD_TO_ATOM().get(&s_real_name) {
+            match keyword_to_atom().get(&s_real_name) {
                 Some(v) => allocator.new_atom(v),
                 None => {
                     let v: Vec<u8> = s_real_name.as_bytes().to_vec();
@@ -75,21 +75,15 @@ pub fn ir_for_atom(atom: &Bytes, allow_keyword: bool) -> IRRepr {
         return IRRepr::Null;
     }
     if atom.length() > 2 {
-        match String::from_utf8(atom.data().to_vec()) {
-            Ok(v) => {
-                if is_printable_string(&v) {
-                    return IRRepr::Quotes(atom.clone());
-                }
+        if let Ok(v) = String::from_utf8(atom.data().to_vec()) {
+            if is_printable_string(&v) {
+                return IRRepr::Quotes(atom.clone());
             }
-            _ => {}
         }
     } else {
         if allow_keyword {
-            match KEYWORD_FROM_ATOM().get(atom.data()) {
-                Some(kw) => {
-                    return IRRepr::Symbol(kw.to_string());
-                }
-                _ => {}
+            if let Some(kw) = keyword_from_atom().get(atom.data()) {
+                return IRRepr::Symbol(kw.to_string());
             }
         }
 
@@ -105,8 +99,8 @@ pub fn ir_for_atom(atom: &Bytes, allow_keyword: bool) -> IRRepr {
 /*
  * (2 2 (2) (2 3 4)) => (a 2 (a) (a 3 4))
  */
-pub fn disassemble_to_ir_with_kw<'a>(
-    allocator: &'a mut Allocator,
+pub fn disassemble_to_ir_with_kw(
+    allocator: &mut Allocator,
     sexp: NodePtr,
     keyword_from_atom: &Record<Vec<u8>, String>,
     allow_keyword_: bool,
@@ -114,16 +108,12 @@ pub fn disassemble_to_ir_with_kw<'a>(
     let mut allow_keyword = allow_keyword_;
     match allocator.sexp(sexp) {
         SExp::Pair(l, r) => {
-            match allocator.sexp(l) {
-                SExp::Pair(_, _) => {
-                    allow_keyword = true;
-                }
-                _ => {}
-            };
+            if let SExp::Pair(_, _) = allocator.sexp(l) {
+                allow_keyword = true;
+            }
 
-            let v0 =
-                disassemble_to_ir_with_kw(allocator, l.clone(), keyword_from_atom, allow_keyword);
-            let v1 = disassemble_to_ir_with_kw(allocator, r.clone(), keyword_from_atom, false);
+            let v0 = disassemble_to_ir_with_kw(allocator, l, keyword_from_atom, allow_keyword);
+            let v1 = disassemble_to_ir_with_kw(allocator, r, keyword_from_atom, false);
             IRRepr::Cons(Rc::new(v0), Rc::new(v1))
         }
 
@@ -134,25 +124,21 @@ pub fn disassemble_to_ir_with_kw<'a>(
     }
 }
 
-pub fn disassemble_with_kw<'a>(
-    allocator: &'a mut Allocator,
+pub fn disassemble_with_kw(
+    allocator: &mut Allocator,
     sexp: NodePtr,
     keyword_from_atom: &Record<Vec<u8>, String>,
 ) -> String {
-    let with_keywords = match allocator.sexp(sexp) {
-        SExp::Atom(_) => false,
-        _ => true,
-    };
-
-    let symbols = disassemble_to_ir_with_kw(allocator, sexp, &keyword_from_atom, with_keywords);
+    let with_keywords = !matches!(allocator.sexp(sexp), SExp::Atom(_));
+    let symbols = disassemble_to_ir_with_kw(allocator, sexp, keyword_from_atom, with_keywords);
     write_ir(Rc::new(symbols))
 }
 
-pub fn disassemble<'a>(allocator: &'a mut Allocator, sexp: NodePtr) -> String {
-    return disassemble_with_kw(allocator, sexp, KEYWORD_FROM_ATOM());
+pub fn disassemble(allocator: &mut Allocator, sexp: NodePtr) -> String {
+    return disassemble_with_kw(allocator, sexp, keyword_from_atom());
 }
 
-pub fn assemble<'a>(allocator: &'a mut Allocator, s: &String) -> Result<NodePtr, EvalErr> {
+pub fn assemble(allocator: &mut Allocator, s: &str) -> Result<NodePtr, EvalErr> {
     let v = s.as_bytes().to_vec();
     let stream = Stream::new(Some(Bytes::new(Some(BytesFromType::Raw(v)))));
     let mut reader = IRReader::new(stream);
