@@ -13,18 +13,17 @@ use std::time::SystemTime;
 
 use core::cmp::max;
 
+use linked_hash_map::LinkedHashMap;
+use yaml_rust::{Yaml, YamlEmitter};
+
 use clvm_rs::allocator::{Allocator, NodePtr};
 use clvm_rs::reduction::EvalErr;
 use clvm_rs::run_program::PreEval;
 
-#[macro_use]
-use yamlette::yamlette;
-use yamlette::model::yaml::str::FORCE_QUOTES;
-
 use crate::classic::clvm::__type_compatibility__::{t, Bytes, BytesFromType, Stream, Tuple};
+use crate::classic::clvm::keyword_from_atom;
 use crate::classic::clvm::serialize::{sexp_from_stream, sexp_to_stream, SimpleCreateCLVMObject};
 use crate::classic::clvm::sexp::{enlist, proper_list, sexp_as_bin};
-use crate::classic::clvm::KEYWORD_FROM_ATOM;
 use crate::classic::clvm_tools::binutils::{assemble_from_ir, disassemble, disassemble_with_kw};
 use crate::classic::clvm_tools::clvmc::detect_modern;
 use crate::classic::clvm_tools::debug::{
@@ -58,7 +57,7 @@ use crate::util::collapse;
 pub struct PathOrCodeConv {}
 
 impl ArgumentValueConv for PathOrCodeConv {
-    fn convert(&self, arg: &String) -> Result<ArgumentValue, String> {
+    fn convert(&self, arg: &str) -> Result<ArgumentValue, String> {
         match fs::read_to_string(arg) {
             Ok(s) => Ok(ArgumentValue::ArgString(Some(arg.to_string()), s)),
             Err(_) => Ok(ArgumentValue::ArgString(None, arg.to_string())),
@@ -76,18 +75,18 @@ pub trait TConversion {
     fn invoke<'a>(
         &self,
         allocator: &'a mut Allocator,
-        text: &String,
+        text: &str,
     ) -> Result<Tuple<NodePtr, String>, String>;
 }
-pub fn call_tool<'a>(
-    allocator: &'a mut Allocator,
-    tool_name: String,
-    desc: String,
+pub fn call_tool(
+    allocator: &mut Allocator,
+    tool_name: &str,
+    desc: &str,
     conversion: Box<dyn TConversion>,
-    input_args: &Vec<String>,
+    input_args: &[String],
 ) {
     let props = TArgumentParserProps {
-        description: desc,
+        description: desc.to_string(),
         prog: tool_name.to_string(),
     };
 
@@ -106,22 +105,15 @@ pub fn call_tool<'a>(
             .set_help("path to clvm script, or literal script".to_string()),
     );
 
-    let mut rest_args = Vec::new();
-    for a in input_args[1..].into_iter() {
-        rest_args.push(a.to_string());
-    }
+    let rest_args: Vec<String> = input_args.iter().skip(1).cloned().collect();
     let args_res = parser.parse_args(&rest_args);
-    let args: HashMap<String, ArgumentValue>;
-
-    match args_res {
-        Ok(a) => {
-            args = a;
-        }
+    let args: HashMap<String, ArgumentValue> = match args_res {
+        Ok(a) => a,
         Err(e) => {
             println!("{}", e);
             return;
         }
-    }
+    };
 
     let args_path_or_code_val = match args.get(&"path_or_code".to_string()) {
         None => ArgumentValue::ArgArray(vec![]),
@@ -143,11 +135,11 @@ pub fn call_tool<'a>(
                 let conv_result = conversion.invoke(allocator, &s);
                 match conv_result {
                     Ok(conv_result) => {
-                        let sexp = conv_result.first().clone();
+                        let sexp = *conv_result.first();
                         let text = conv_result.rest();
                         if args.contains_key(&"script_hash".to_string()) {
                             println!("{}", sha256tree(allocator, sexp).hex());
-                        } else if text.len() > 0 {
+                        } else if !text.is_empty() {
                             println!("{}", text);
                         }
                     }
@@ -169,7 +161,7 @@ impl TConversion for OpcConversion {
     fn invoke<'a>(
         &self,
         allocator: &'a mut Allocator,
-        hex_text: &String,
+        hex_text: &str,
     ) -> Result<Tuple<NodePtr, String>, String> {
         read_ir(hex_text)
             .and_then(|ir_sexp| assemble_from_ir(allocator, Rc::new(ir_sexp)).map_err(|e| e.1))
@@ -183,7 +175,7 @@ impl TConversion for OpdConversion {
     fn invoke<'a>(
         &self,
         allocator: &'a mut Allocator,
-        hex_text: &String,
+        hex_text: &str,
     ) -> Result<Tuple<NodePtr, String>, String> {
         let mut stream = Stream::new(Some(Bytes::new(Some(BytesFromType::Hex(
             hex_text.to_string(),
@@ -198,23 +190,23 @@ impl TConversion for OpdConversion {
     }
 }
 
-pub fn opc(args: &Vec<String>) {
+pub fn opc(args: &[String]) {
     let mut allocator = Allocator::new();
     call_tool(
         &mut allocator,
-        "opc".to_string(),
-        "Compile a clvm script.".to_string(),
+        "opc",
+        "Compile a clvm script.",
         Box::new(OpcConversion {}),
         args,
     );
 }
 
-pub fn opd(args: &Vec<String>) {
+pub fn opd(args: &[String]) {
     let mut allocator = Allocator::new();
     call_tool(
         &mut allocator,
-        "opd".to_string(),
-        "Disassemble a compiled clvm script from hex.".to_string(),
+        "opd",
+        "Disassemble a compiled clvm script from hex.",
         Box::new(OpdConversion {}),
         args,
     );
@@ -223,7 +215,7 @@ pub fn opd(args: &Vec<String>) {
 struct StageImport {}
 
 impl ArgumentValueConv for StageImport {
-    fn convert(&self, arg: &String) -> Result<ArgumentValue, String> {
+    fn convert(&self, arg: &str) -> Result<ArgumentValue, String> {
         if arg == "0" {
             return Ok(ArgumentValue::ArgInt(0));
         } else if arg == "1" {
@@ -231,27 +223,41 @@ impl ArgumentValueConv for StageImport {
         } else if arg == "2" {
             return Ok(ArgumentValue::ArgInt(2));
         }
-        return Err(format!("Unknown stage: {}", arg));
+        Err(format!("Unknown stage: {}", arg))
     }
 }
 
-pub fn run(args: &Vec<String>) {
+pub fn run(args: &[String]) {
     let mut s = Stream::new(None);
-    launch_tool(&mut s, args, &"run".to_string(), 2);
+    launch_tool(&mut s, args, "run", 2);
     io::stdout()
         .write_all(s.get_value().data())
         .expect("stdout");
 }
 
-pub fn brun(args: &Vec<String>) {
+pub fn brun(args: &[String]) {
     let mut s = Stream::new(None);
-    launch_tool(&mut s, args, &"brun".to_string(), 0);
+    launch_tool(&mut s, args, "brun", 0);
     io::stdout()
         .write_all(s.get_value().data())
         .expect("stdout");
 }
 
-pub fn cldb(args: &Vec<String>) {
+fn to_yaml(entries: &[BTreeMap<String, String>]) -> Yaml {
+    let result_array: Vec<Yaml> = entries
+        .iter()
+        .map(|tm| {
+            let mut h = LinkedHashMap::new();
+            for (k, v) in tm.iter() {
+                h.insert(Yaml::String(k.clone()), Yaml::String(v.clone()));
+            }
+            Yaml::Hash(h)
+        })
+        .collect();
+    Yaml::Array(result_array)
+}
+
+pub fn cldb(args: &[String]) {
     let tool_name = "cldb".to_string();
     let props = TArgumentParserProps {
         description: "Execute a clvm script.".to_string(),
@@ -300,73 +306,57 @@ pub fn cldb(args: &Vec<String>) {
             .set_help("clvm script environment, as clvm src, or hex".to_string()),
     );
     let arg_vec = args[1..].to_vec();
-    let parsedArgs: HashMap<String, ArgumentValue>;
 
     let mut input_file = None;
     let mut input_program = "()".to_string();
 
-    let prog_srcloc = Srcloc::start(&"*program*".to_string());
-    let args_srcloc = Srcloc::start(&"*args*".to_string());
+    let prog_srcloc = Srcloc::start("*program*");
+    let args_srcloc = Srcloc::start("*args*");
 
-    let mut args = Rc::new(sexp::SExp::atom_from_string(
-        args_srcloc.clone(),
-        &"".to_string(),
-    ));
+    let mut args = Rc::new(sexp::SExp::atom_from_string(args_srcloc.clone(), ""));
     let mut parsed_args_result: String = "".to_string();
 
-    match parser.parse_args(&arg_vec) {
+    let parsed_args: HashMap<String, ArgumentValue> = match parser.parse_args(&arg_vec) {
         Err(e) => {
             println!("FAIL: {}", e);
             return;
         }
-        Ok(pa) => {
-            parsedArgs = pa;
-        }
-    }
+        Ok(pa) => pa,
+    };
 
-    if let Some(ArgumentValue::ArgArray(v)) = parsedArgs.get("include") {
+    if let Some(ArgumentValue::ArgArray(v)) = parsed_args.get("include") {
         for p in v {
-            match p {
-                ArgumentValue::ArgString(_, s) => search_paths.push(s.to_string()),
-                _ => {}
+            if let ArgumentValue::ArgString(_, s) = p {
+                search_paths.push(s.to_string());
             }
         }
     }
 
-    match parsedArgs.get("path_or_code") {
-        Some(ArgumentValue::ArgString(file, path_or_code)) => {
-            input_file = file.clone();
-            input_program = path_or_code.to_string();
-        }
-        _ => {}
+    if let Some(ArgumentValue::ArgString(file, path_or_code)) = parsed_args.get("path_or_code") {
+        input_file = file.clone();
+        input_program = path_or_code.to_string();
     }
 
-    match parsedArgs.get("env") {
-        Some(ArgumentValue::ArgString(_, s)) => {
-            parsed_args_result = s.to_string();
-        }
-        _ => {}
+    if let Some(ArgumentValue::ArgString(_, s)) = parsed_args.get("env") {
+        parsed_args_result = s.to_string();
     }
 
     let mut allocator = Allocator::new();
 
-    let symbol_table = parsedArgs
+    let symbol_table = parsed_args
         .get("symbol_table")
         .and_then(|jstring| match jstring {
             ArgumentValue::ArgString(_, s) => {
                 let decoded_symbol_table: Option<HashMap<String, String>> =
-                    serde_json::from_str(&s).ok();
+                    serde_json::from_str(s).ok();
                 decoded_symbol_table
             }
             _ => None,
         });
 
-    let do_optimize = parsedArgs
+    let do_optimize = parsed_args
         .get("optimize")
-        .map(|x| match x {
-            ArgumentValue::ArgBool(true) => true,
-            _ => false,
-        })
+        .map(|x| matches!(x, ArgumentValue::ArgBool(true)))
         .unwrap_or_else(|| false);
     let runner = Rc::new(DefaultProgramRunner::new());
     let use_filename = input_file
@@ -376,7 +366,7 @@ pub fn cldb(args: &Vec<String>) {
         .set_optimize(do_optimize)
         .set_search_paths(&search_paths);
 
-    let mut use_symbol_table = symbol_table.unwrap_or_else(|| HashMap::new());
+    let mut use_symbol_table = symbol_table.unwrap_or_default();
     let unopt_res = compile_file(
         &mut allocator,
         runner.clone(),
@@ -385,15 +375,17 @@ pub fn cldb(args: &Vec<String>) {
         &mut use_symbol_table,
     );
 
-    let program;
     let mut output = Vec::new();
-    let yamlette_string = |to_print: Vec<BTreeMap<String, String>>| match yamlette!(write; [[( # FORCE_QUOTES => to_print )]])
-    {
-        Ok(s) => s,
-        Err(e) => format!("error producing yaml: {:?}", e),
+    let yamlette_string = |to_print: Vec<BTreeMap<String, String>>| {
+        let mut result = String::new();
+        let mut emitter = YamlEmitter::new(&mut result);
+        match emitter.dump(&to_yaml(&to_print)) {
+            Ok(_) => result,
+            Err(e) => format!("error producing yaml: {:?}", e),
+        }
     };
 
-    let res = match parsedArgs.get("hex") {
+    let res = match parsed_args.get("hex") {
         Some(ArgumentValue::ArgBool(true)) => hex_to_modern_sexp(
             &mut allocator,
             &use_symbol_table,
@@ -405,15 +397,13 @@ pub fn cldb(args: &Vec<String>) {
             if do_optimize {
                 unopt_res.and_then(|x| run_optimizer(&mut allocator, runner.clone(), Rc::new(x)))
             } else {
-                unopt_res.map(|x| Rc::new(x))
+                unopt_res.map(Rc::new)
             }
         }
     };
 
-    match res {
-        Ok(r) => {
-            program = r.clone();
-        }
+    let program = match res {
+        Ok(r) => r,
         Err(c) => {
             let mut parse_error = BTreeMap::new();
             parse_error.insert("Error-Location".to_string(), c.0.to_string());
@@ -422,14 +412,14 @@ pub fn cldb(args: &Vec<String>) {
             println!("{}", yamlette_string(output));
             return;
         }
-    }
+    };
 
-    match parsedArgs.get("hex") {
+    match parsed_args.get("hex") {
         Some(ArgumentValue::ArgBool(true)) => {
             match hex_to_modern_sexp(
                 &mut allocator,
                 &HashMap::new(),
-                args_srcloc.clone(),
+                args_srcloc,
                 &parsed_args_result,
             ) {
                 Ok(r) => {
@@ -444,9 +434,9 @@ pub fn cldb(args: &Vec<String>) {
                 }
             }
         }
-        _ => match parse_sexp(Srcloc::start(&"*arg*".to_string()), &parsed_args_result) {
+        _ => match parse_sexp(Srcloc::start("*arg*"), &parsed_args_result) {
             Ok(r) => {
-                if r.len() > 0 {
+                if !r.is_empty() {
                     args = r[0].clone();
                 }
             }
@@ -466,7 +456,7 @@ pub fn cldb(args: &Vec<String>) {
         prim_map.insert(p.0.clone(), Rc::new(p.1.clone()));
     }
     let program_lines: Vec<String> = input_program.lines().map(|x| x.to_string()).collect();
-    let step = start_step(program.clone(), args.clone());
+    let step = start_step(program, args);
     let cldbenv = CldbRunEnv::new(
         input_file,
         program_lines,
@@ -480,11 +470,8 @@ pub fn cldb(args: &Vec<String>) {
             return;
         }
 
-        match cldbrun.step(&mut allocator) {
-            Some(result) => {
-                output.push(result);
-            }
-            _ => {}
+        if let Some(result) = cldbrun.step(&mut allocator) {
+            output.push(result);
         }
     }
 }
@@ -526,7 +513,7 @@ fn calculate_cost_offset(
      This is a hack and need to go away, probably when we do dialects for real,
      and then the dialect can have a `run_program` API.
     */
-    let almost_empty_list = enlist(allocator, &vec![allocator.null()]).unwrap();
+    let almost_empty_list = enlist(allocator, &[allocator.null()]).unwrap();
     let cost = run_program
         .run_program(allocator, run_script, almost_empty_list, None)
         .map(|x| x.0)
@@ -537,18 +524,17 @@ fn calculate_cost_offset(
 
 fn fix_log(
     allocator: &mut Allocator,
-    log_result: &mut Vec<NodePtr>,
-    log_updates: &Vec<(NodePtr, Option<NodePtr>)>,
+    log_result: &mut [NodePtr],
+    log_updates: &[(NodePtr, Option<NodePtr>)],
 ) {
     let mut update_map: HashMap<NodePtr, Option<NodePtr>> = HashMap::new();
     for update in log_updates {
         update_map.insert(update.0, update.1);
     }
 
-    for i in 0..log_result.len() {
-        let entry = log_result[i];
-        update_map.get(&entry).and_then(|v| *v).map(|v| {
-            proper_list(allocator, entry, true).map(|list| {
+    for (i, entry) in log_result.to_vec().iter().enumerate() {
+        update_map.get(entry).and_then(|v| *v).map(|v| {
+            proper_list(allocator, *entry, true).map(|list| {
                 let mut updated = list.to_vec();
                 updated.push(v);
                 log_result[i] = enlist(allocator, &updated).unwrap();
@@ -557,27 +543,19 @@ fn fix_log(
     }
 }
 
-fn write_sym_output(
-    compiled_lookup: &HashMap<String, String>,
-    path: &String,
-) -> Result<(), String> {
+fn write_sym_output(compiled_lookup: &HashMap<String, String>, path: &str) -> Result<(), String> {
     m! {
         output <- serde_json::to_string(compiled_lookup).map_err(|_| {
             "failed to serialize to json".to_string()
         });
 
-        fs::write(path.clone(), output).map_err(|_| {
+        fs::write(path, output).map_err(|_| {
             format!("failed to write {}", path)
         }).map(|_| ())
     }
 }
 
-pub fn launch_tool(
-    stdout: &mut Stream,
-    args: &Vec<String>,
-    tool_name: &String,
-    default_stage: u32,
-) {
+pub fn launch_tool(stdout: &mut Stream, args: &[String], tool_name: &str, default_stage: u32) {
     let props = TArgumentParserProps {
         description: "Execute a clvm script.".to_string(),
         prog: format!("clvm_tools {}", tool_name),
@@ -685,6 +663,12 @@ pub fn launch_tool(
             .set_action(TArgOptionAction::StoreTrue)
             .set_help("run optimizer".to_string()),
     );
+    parser.add_argument(
+        vec!["--only-exn".to_string()],
+        Argument::new()
+            .set_action(TArgOptionAction::StoreTrue)
+            .set_help("Only show frames along the exception path".to_string()),
+    );
 
     if tool_name == "run" {
         parser.add_argument(
@@ -698,35 +682,30 @@ pub fn launch_tool(
     }
 
     let arg_vec = args[1..].to_vec();
-    let parsedArgs: HashMap<String, ArgumentValue>;
-
-    match parser.parse_args(&arg_vec) {
+    let parsed_args: HashMap<String, ArgumentValue> = match parser.parse_args(&arg_vec) {
         Err(e) => {
-            stdout.write_string(format!("FAIL: {}\n", e));
+            stdout.write_str(&format!("FAIL: {}\n", e));
             return;
         }
-        Ok(pa) => {
-            parsedArgs = pa;
-        }
-    }
+        Ok(pa) => pa,
+    };
 
     let empty_map = HashMap::new();
-    let keywords = match parsedArgs.get("no_keywords") {
-        None => KEYWORD_FROM_ATOM(),
+    let keywords = match parsed_args.get("no_keywords") {
+        None => keyword_from_atom(),
         Some(ArgumentValue::ArgBool(_b)) => &empty_map,
-        _ => KEYWORD_FROM_ATOM(),
+        _ => keyword_from_atom(),
     };
 
     let dpr;
     let run_program: Rc<dyn TRunProgram>;
     let mut search_paths = Vec::new();
-    match parsedArgs.get("include") {
+    match parsed_args.get("include") {
         Some(ArgumentValue::ArgArray(v)) => {
             let mut bare_paths = Vec::with_capacity(v.len());
             for p in v {
-                match p {
-                    ArgumentValue::ArgString(_, s) => bare_paths.push(s.to_string()),
-                    _ => {}
+                if let ArgumentValue::ArgString(_, s) = p {
+                    bare_paths.push(s.to_string());
                 }
             }
             let special_runner = run_program_for_search_paths(&bare_paths);
@@ -744,89 +723,91 @@ pub fn launch_tool(
     let mut allocator = Allocator::new();
 
     let mut input_file = None;
-    let mut input_serialized = None;
-    let mut input_sexp;
+    let input_serialized = None;
+    let mut input_sexp: Option<NodePtr> = None;
 
     let time_start = SystemTime::now();
     let mut time_read_hex = SystemTime::now();
     let mut time_assemble = SystemTime::now();
-    let time_parse_input;
 
     let mut input_program = "()".to_string();
     let mut input_args = "()".to_string();
 
-    match parsedArgs.get("path_or_code") {
-        Some(ArgumentValue::ArgString(file, path_or_code)) => {
-            input_file = file.clone();
-            input_program = path_or_code.to_string();
-        }
-        _ => {}
+    if let Some(ArgumentValue::ArgString(file, path_or_code)) = parsed_args.get("path_or_code") {
+        input_file = file.clone();
+        input_program = path_or_code.to_string();
     }
 
-    match parsedArgs.get("hex") {
+    if let Some(ArgumentValue::ArgString(file, path_or_code)) = parsed_args.get("env") {
+        input_file = file.clone();
+        input_args = path_or_code.to_string();
+    }
+
+    match parsed_args.get("hex") {
         Some(_) => {
             let assembled_serialized =
                 Bytes::new(Some(BytesFromType::Hex(input_program.to_string())));
-            if input_args.len() == 0 {
-                input_args = "80".to_string();
-            }
 
-            let env_serialized = Bytes::new(Some(BytesFromType::Hex(input_args.to_string())));
+            let env_serialized = if input_args.is_empty() {
+                Bytes::new(Some(BytesFromType::Hex("80".to_string())))
+            } else {
+                Bytes::new(Some(BytesFromType::Hex(input_args)))
+            };
+
             time_read_hex = SystemTime::now();
 
-            input_serialized = Some(
-                Bytes::new(Some(BytesFromType::Raw(vec![0xff])))
-                    .concat(&assembled_serialized)
-                    .concat(&env_serialized),
-            );
-
-            let mut stream = Stream::new(input_serialized.clone());
-            input_sexp = sexp_from_stream(
+            let mut prog_stream = Stream::new(Some(assembled_serialized));
+            let input_prog_sexp = sexp_from_stream(
                 &mut allocator,
-                &mut stream,
+                &mut prog_stream,
                 Box::new(SimpleCreateCLVMObject {}),
             )
             .map(|x| Some(x.1))
             .unwrap();
+
+            let mut arg_stream = Stream::new(Some(env_serialized));
+            let input_arg_sexp = sexp_from_stream(
+                &mut allocator,
+                &mut arg_stream,
+                Box::new(SimpleCreateCLVMObject {}),
+            )
+            .map(|x| Some(x.1))
+            .unwrap();
+            if let (Some(ip), Some(ia)) = (input_prog_sexp, input_arg_sexp) {
+                input_sexp = allocator.new_pair(ip, ia).ok();
+            }
         }
         _ => {
             let src_sexp;
-            match parsedArgs.get("path_or_code") {
-                Some(ArgumentValue::ArgString(f, content)) => match read_ir(&content) {
+            if let Some(ArgumentValue::ArgString(f, content)) = parsed_args.get("path_or_code") {
+                match read_ir(content) {
                     Ok(s) => {
                         input_program = content.clone();
                         input_file = f.clone();
                         src_sexp = s;
                     }
                     Err(e) => {
-                        stdout.write_string(format!("FAIL: {}\n", e));
+                        stdout.write_str(&format!("FAIL: {}\n", e));
                         return;
                     }
-                },
-                _ => {
-                    stdout.write_string(format!("FAIL: {}\n", "non-string argument"));
-                    return;
                 }
+            } else {
+                stdout.write_str(&format!("FAIL: {}\n", "non-string argument"));
+                return;
             }
 
             let assembled_sexp = assemble_from_ir(&mut allocator, Rc::new(src_sexp)).unwrap();
             let mut parsed_args_result = "()".to_string();
 
-            match parsedArgs.get("env") {
-                Some(ArgumentValue::ArgString(_f, s)) => {
-                    parsed_args_result = s.to_string();
-                }
-                _ => {}
+            if let Some(ArgumentValue::ArgString(_f, s)) = parsed_args.get("env") {
+                parsed_args_result = s.to_string();
             }
 
             let env_ir = read_ir(&parsed_args_result).unwrap();
             let env = assemble_from_ir(&mut allocator, Rc::new(env_ir)).unwrap();
             time_assemble = SystemTime::now();
 
-            input_sexp = allocator
-                .new_pair(assembled_sexp, env)
-                .map(|x| Some(x))
-                .unwrap();
+            input_sexp = allocator.new_pair(assembled_sexp, env).ok();
         }
     }
 
@@ -835,7 +816,7 @@ pub fn launch_tool(
     let mut symbol_table: Option<HashMap<String, String>> = None;
     let mut emit_symbol_output = false;
 
-    let symbol_table_clone = parsedArgs
+    let symbol_table_clone = parsed_args
         .get("symbol_table")
         .and_then(|jstring| match jstring {
             ArgumentValue::ArgString(_, s) => fs::read_to_string(s).ok().and_then(|s| {
@@ -851,35 +832,29 @@ pub fn launch_tool(
             st
         });
 
-    match parsedArgs.get("verbose") {
-        Some(ArgumentValue::ArgBool(true)) => {
-            emit_symbol_output = true;
-        }
-        _ => {}
+    if let Some(ArgumentValue::ArgBool(true)) = parsed_args.get("verbose") {
+        emit_symbol_output = true;
     }
 
     // Add unused check.
-    let do_check_unused = parsedArgs
+    let do_check_unused = parsed_args
         .get("check_unused_args")
-        .map(|a| match a {
-            ArgumentValue::ArgBool(true) => true,
-            _ => false,
-        })
-        .unwrap_or_else(|| false);
+        .map(|a| matches!(a, ArgumentValue::ArgBool(true)))
+        .unwrap_or(false);
 
     let dialect = input_sexp.and_then(|i| detect_modern(&mut allocator, i));
-    let mut stderr_output = |s| {
-        if let Some(_) = dialect {
+    let mut stderr_output = |s: String| {
+        if dialect.is_some() {
             eprintln!("{}", s);
         } else {
-            stdout.write_string(s);
+            stdout.write_str(&s);
         }
     };
 
     if do_check_unused {
         let use_filename = input_file
             .as_ref()
-            .map(|x| x.clone())
+            .cloned()
             .unwrap_or_else(|| "*command*".to_string());
         let opts = Rc::new(DefaultCompilerOpts::new(&use_filename)).set_search_paths(&search_paths);
         match check_unused(opts, &input_program) {
@@ -890,7 +865,7 @@ pub fn launch_tool(
                 }
             }
             Err(e) => {
-                stderr_output(format!("{}: {}\n", e.0.to_string(), e.1));
+                stderr_output(format!("{}: {}\n", e.0, e.1));
                 return;
             }
         }
@@ -898,12 +873,9 @@ pub fn launch_tool(
 
     // In testing: short circuit for modern compilation.
     if let Some(dialect) = dialect {
-        let do_optimize = parsedArgs
+        let do_optimize = parsed_args
             .get("optimize")
-            .map(|x| match x {
-                ArgumentValue::ArgBool(true) => true,
-                _ => false,
-            })
+            .map(|x| matches!(x, ArgumentValue::ArgBool(true)))
             .unwrap_or_else(|| false);
         let runner = Rc::new(DefaultProgramRunner::new());
         let use_filename = input_file.unwrap_or_else(|| "*command*".to_string());
@@ -923,18 +895,18 @@ pub fn launch_tool(
         let res = if do_optimize {
             unopt_res.and_then(|x| run_optimizer(&mut allocator, runner, Rc::new(x)))
         } else {
-            unopt_res.map(|x| Rc::new(x))
+            unopt_res.map(Rc::new)
         };
 
         match res {
             Ok(r) => {
-                stdout.write_string(r.to_string());
+                stdout.write_str(&r.to_string());
 
                 build_symbol_table_mut(&mut symbol_table, &r);
-                write_sym_output(&symbol_table, &"main.sym".to_string()).expect("writing symbols");
+                write_sym_output(&symbol_table, "main.sym").expect("writing symbols");
             }
             Err(c) => {
-                stdout.write_string(format!("{}: {}", c.0.to_string(), c.1));
+                stdout.write_str(&format!("{}: {}", c.0, c.1));
             }
         }
 
@@ -947,6 +919,7 @@ pub fn launch_tool(
     let log_entries: Arc<Mutex<RunLog<NodePtr>>> = Arc::new(Mutex::new(RunLog {
         log_entries: RefCell::new(Vec::new()),
     }));
+    #[allow(clippy::type_complexity)]
     let log_updates: Arc<Mutex<RunLog<(NodePtr, Option<NodePtr>)>>> =
         Arc::new(Mutex::new(RunLog {
             log_entries: RefCell::new(Vec::new()),
@@ -963,15 +936,17 @@ pub fn launch_tool(
     let (post_eval_resp_out, post_eval_resp_in): (Sender<()>, Receiver<()>) = channel();
 
     let post_eval_fn: Rc<dyn Fn(NodePtr, Option<NodePtr>)> = Rc::new(move |at, n| {
-        post_eval_req_out.send((at, n));
+        post_eval_req_out.send((at, n)).ok();
         post_eval_resp_in.recv().unwrap();
     });
 
+    #[allow(clippy::type_complexity)]
     let pre_eval_fn: Rc<dyn Fn(&mut Allocator, NodePtr)> = Rc::new(move |_allocator, new_log| {
-        pre_eval_req_out.send(new_log);
+        pre_eval_req_out.send(new_log).ok();
         pre_eval_resp_in.recv().unwrap();
     });
 
+    #[allow(clippy::type_complexity)]
     let closure: Rc<dyn Fn(NodePtr) -> Box<dyn Fn(Option<NodePtr>)>> = Rc::new(move |v| {
         let post_eval_fn_clone = post_eval_fn.clone();
         Box::new(move |n| {
@@ -981,6 +956,7 @@ pub fn launch_tool(
     });
 
     if emit_symbol_output {
+        #[allow(clippy::type_complexity)]
         let pre_eval_f_closure: Box<
             dyn Fn(
                 &mut Allocator,
@@ -1007,15 +983,14 @@ pub fn launch_tool(
         pre_eval_f = Some(pre_eval_f_closure);
     }
 
-    let run_script = match parsedArgs.get("stage") {
+    let run_script = match parsed_args.get("stage") {
         Some(ArgumentValue::ArgInt(0)) => stages::brun(&mut allocator),
         _ => stages::run(&mut allocator),
     };
 
-    let mut output = "(didn't finish)".to_string();
     let cost_offset = calculate_cost_offset(&mut allocator, run_program.clone(), run_script);
 
-    let max_cost = parsedArgs
+    let max_cost = parsed_args
         .get("max_cost")
         .map(|x| match x {
             ArgumentValue::ArgInt(i) => *i as i64 - cost_offset,
@@ -1027,7 +1002,7 @@ pub fn launch_tool(
     if input_sexp.is_none() {
         input_sexp = sexp_from_stream(
             &mut allocator,
-            &mut Stream::new(input_serialized.clone()),
+            &mut Stream::new(input_serialized),
             Box::new(SimpleCreateCLVMObject {}),
         )
         .map(|x| Some(x.1))
@@ -1041,19 +1016,12 @@ pub fn launch_tool(
         let pre_in = pre_eval_req_in;
         let pre_out = pre_eval_resp_out;
 
-        loop {
-            match pre_in.recv() {
-                Ok(received) => {
-                    {
-                        let locked = log_entries_clone.lock();
-                        locked.unwrap().push(received);
-                    }
-                    pre_out.send(());
-                }
-                Err(_e) => {
-                    break;
-                }
+        while let Ok(received) = pre_in.recv() {
+            {
+                let locked = log_entries_clone.lock();
+                locked.unwrap().push(received);
             }
+            pre_out.send(()).ok();
         }
     });
 
@@ -1062,24 +1030,16 @@ pub fn launch_tool(
         let post_in = post_eval_req_in;
         let post_out = post_eval_resp_out;
 
-        loop {
-            match post_in.recv() {
-                Ok(received) => {
-                    {
-                        let locked = log_updates_clone.lock();
-                        locked.unwrap().push(received);
-                    }
-                    post_out.send(());
-                }
-                Err(_e) => {
-                    break;
-                }
+        while let Ok(received) = post_in.recv() {
+            {
+                let locked = log_updates_clone.lock();
+                locked.unwrap().push(received);
             }
+            post_out.send(()).ok();
         }
     });
 
-    time_parse_input = SystemTime::now();
-
+    let time_parse_input = SystemTime::now();
     let res = run_program
         .run_program(
             &mut allocator,
@@ -1092,7 +1052,7 @@ pub fn launch_tool(
                     Some(max_cost as u64)
                 },
                 pre_eval_f,
-                strict: parsedArgs
+                strict: parsed_args
                     .get("strict")
                     .map(|_| true)
                     .unwrap_or_else(|| false),
@@ -1103,69 +1063,58 @@ pub fn launch_tool(
             let result = run_program_result.1;
             let time_done = SystemTime::now();
 
-            let _ = if !parsedArgs.get("cost").is_none() {
+            if parsed_args.get("cost").is_some() {
                 if cost > 0 {
                     cost += cost_offset;
                 }
-                stdout.write_string(format!("cost = {}\n", cost));
+                stdout.write_str(&format!("cost = {}\n", cost));
             };
 
-            let _ = match parsedArgs.get("time") {
-                Some(ArgumentValue::ArgInt(_t)) => {
-                    match parsedArgs.get("hex") {
-                        Some(_) => {
-                            stdout.write_string(format!(
-                                "read_hex: {}\n",
-                                time_read_hex
-                                    .duration_since(time_start)
-                                    .unwrap()
-                                    .as_millis()
-                            ));
-                        }
-                        _ => {
-                            stdout.write_string(format!(
-                                "assemble_from_ir: {}\n",
-                                time_assemble
-                                    .duration_since(time_start)
-                                    .unwrap()
-                                    .as_millis()
-                            ));
-                            stdout.write_string(format!(
-                                "to_sexp_f: {}\n",
-                                time_parse_input
-                                    .duration_since(time_assemble)
-                                    .unwrap()
-                                    .as_millis()
-                            ));
-                        }
-                    }
-                    stdout.write_string(format!(
-                        "run_program: {}\n",
-                        time_done
-                            .duration_since(time_parse_input)
+            if let Some(ArgumentValue::ArgBool(true)) = parsed_args.get("time") {
+                if parsed_args.get("hex").is_some() {
+                    stdout.write_str(&format!(
+                        "read_hex: {}\n",
+                        time_read_hex
+                            .duration_since(time_start)
+                            .unwrap()
+                            .as_millis()
+                    ));
+                } else {
+                    stdout.write_str(&format!(
+                        "assemble_from_ir: {}\n",
+                        time_assemble
+                            .duration_since(time_start)
+                            .unwrap()
+                            .as_millis()
+                    ));
+                    stdout.write_str(&format!(
+                        "to_sexp_f: {}\n",
+                        time_parse_input
+                            .duration_since(time_assemble)
                             .unwrap()
                             .as_millis()
                     ));
                 }
-                _ => {}
+
+                stdout.write_str(&format!(
+                    "run_program: {}\n",
+                    time_done
+                        .duration_since(time_parse_input)
+                        .unwrap()
+                        .as_millis()
+                ));
+            }
+
+            let mut run_output = disassemble_with_kw(&mut allocator, result, keywords);
+            if let Some(ArgumentValue::ArgBool(true)) = parsed_args.get("dump") {
+                let mut f = Stream::new(None);
+                sexp_to_stream(&mut allocator, result, &mut f);
+                run_output = f.get_value().hex();
+            } else if let Some(ArgumentValue::ArgBool(true)) = parsed_args.get("quiet") {
+                run_output = "".to_string();
             };
 
-            let _ = output = disassemble_with_kw(&mut allocator, result, keywords);
-            let _ = match parsedArgs.get("dump") {
-                Some(ArgumentValue::ArgBool(true)) => {
-                    let mut f = Stream::new(None);
-                    sexp_to_stream(&mut allocator, result, &mut f);
-                    output = f.get_value().hex();
-                }
-                _ => match parsedArgs.get("quiet") {
-                    Some(ArgumentValue::ArgBool(true)) => {
-                        output = "".to_string();
-                    }
-                    _ => {}
-                },
-            };
-
-            output
+            run_output
         });
 
     let output = collapse(res.map_err(|ex| {
@@ -1177,11 +1126,11 @@ pub fn launch_tool(
     }));
 
     let compile_sym_out = dpr.get_compiles();
-    if compile_sym_out.len() > 0 {
-        write_sym_output(&compile_sym_out, &"main.sym".to_string());
+    if !compile_sym_out.is_empty() {
+        write_sym_output(&compile_sym_out, "main.sym").ok();
     }
 
-    stdout.write_string(format!("{}\n", output));
+    stdout.write_str(&format!("{}\n", output));
 
     // Third part of our scheme: now that we have results from the forward pass
     // and the pass doing the post callbacks, we can integrate them in the main
@@ -1191,20 +1140,27 @@ pub fn launch_tool(
     let log_updates = log_updates.lock().unwrap().finish();
     fix_log(&mut allocator, &mut log_content, &log_updates);
 
+    let only_exn = parsed_args
+        .get("only_exn")
+        .map(|_| true)
+        .unwrap_or_else(|| false);
+
     if emit_symbol_output {
-        stdout.write_string(format!("\n"));
+        stdout.write_str("\n");
         trace_to_text(
             &mut allocator,
             stdout,
+            only_exn,
             &log_content,
             symbol_table.clone(),
             &disassemble,
         );
-        if !parsedArgs.get("table").is_none() {
+        if parsed_args.get("table").is_some() {
             trace_to_table(
                 &mut allocator,
                 stdout,
-                &mut log_content,
+                only_exn,
+                &log_content,
                 symbol_table,
                 &disassemble,
             );
