@@ -1,4 +1,4 @@
-pub mod Type;
 pub mod reader;
+pub mod r#type;
 pub mod utils;
 pub mod writer;
