@@ -2,7 +2,7 @@ use clvm_rs::allocator::{Allocator, NodePtr, SExp};
 
 use crate::classic::clvm::__type_compatibility__::{sha256, Bytes, BytesFromType};
 
-pub fn sha256tree<'a>(allocator: &'a mut Allocator, v: NodePtr) -> Bytes {
+pub fn sha256tree(allocator: &mut Allocator, v: NodePtr) -> Bytes {
     match allocator.sexp(v) {
         SExp::Pair(l, r) => {
             let left = sha256tree(allocator, l);
