@@ -9,7 +9,7 @@ use clvm_rs::allocator::Allocator;
 use crate::classic::clvm::__type_compatibility__::{bi_one, bi_zero};
 use crate::classic::clvm_tools::stages::stage_0::TRunProgram;
 
-use crate::compiler::clvm::{run, sha256tree};
+use crate::compiler::clvm::run;
 use crate::compiler::compiler::is_at_capture;
 use crate::compiler::comptypes::{
     Binding, BodyForm, CompileErr, CompileForm, CompilerOpts, HelperForm, LetFormKind,
@@ -37,7 +37,7 @@ pub struct Evaluator {
     ignore_exn: bool,
 }
 
-fn select_helper(bindings: &Vec<HelperForm>, name: &Vec<u8>) -> Option<HelperForm> {
+fn select_helper(bindings: &[HelperForm], name: &[u8]) -> Option<HelperForm> {
     for b in bindings.iter() {
         if b.name() == name {
             return Some(b.clone());
@@ -49,7 +49,7 @@ fn select_helper(bindings: &Vec<HelperForm>, name: &Vec<u8>) -> Option<HelperFor
 
 fn update_parallel_bindings(
     bindings: &HashMap<Vec<u8>, Rc<BodyForm>>,
-    have_bindings: &Vec<Rc<Binding>>,
+    have_bindings: &[Rc<Binding>],
 ) -> HashMap<Vec<u8>, Rc<BodyForm>> {
     let mut new_bindings = bindings.clone();
     for b in have_bindings.iter() {
@@ -60,13 +60,13 @@ fn update_parallel_bindings(
 
 // Tell whether the bodyform is a simple primitive.
 pub fn is_primitive(expr: &BodyForm) -> bool {
-    match expr {
-        BodyForm::Quoted(_) => true,
-        BodyForm::Value(SExp::Nil(_)) => true,
-        BodyForm::Value(SExp::Integer(_, _)) => true,
-        BodyForm::Value(SExp::QuotedString(_, _, _)) => true,
-        _ => false,
-    }
+    matches!(
+        expr,
+        BodyForm::Quoted(_)
+            | BodyForm::Value(SExp::Nil(_))
+            | BodyForm::Value(SExp::Integer(_, _))
+            | BodyForm::Value(SExp::QuotedString(_, _, _))
+    )
 }
 
 fn make_operator1(l: &Srcloc, op: String, arg: Rc<BodyForm>) -> BodyForm {
@@ -74,7 +74,7 @@ fn make_operator1(l: &Srcloc, op: String, arg: Rc<BodyForm>) -> BodyForm {
         l.clone(),
         vec![
             Rc::new(BodyForm::Value(SExp::atom_from_string(l.clone(), &op))),
-            arg.clone(),
+            arg,
         ],
     )
 }
@@ -84,8 +84,8 @@ fn make_operator2(l: &Srcloc, op: String, arg1: Rc<BodyForm>, arg2: Rc<BodyForm>
         l.clone(),
         vec![
             Rc::new(BodyForm::Value(SExp::atom_from_string(l.clone(), &op))),
-            arg1.clone(),
-            arg2.clone(),
+            arg1,
+            arg2,
         ],
     )
 }
@@ -98,7 +98,7 @@ fn get_bodyform_from_arginput(l: &Srcloc, arginput: &ArgInputs) -> Rc<BodyForm>
         ArgInputs::Pair(a, b) => {
             let bfa = get_bodyform_from_arginput(l, a);
             let bfb = get_bodyform_from_arginput(l, b);
-            Rc::new(make_operator2(l, "c".to_string(), bfa.clone(), bfb.clone()))
+            Rc::new(make_operator2(l, "c".to_string(), bfa, bfb))
         }
     }
 }
@@ -119,7 +119,7 @@ fn create_argument_captures(
         (ArgInputs::Whole(bf), SExp::Cons(l, f, r)) => {
             match (is_at_capture(f.clone(), r.clone()), bf.borrow()) {
                 (Some((capture, substructure)), BodyForm::Quoted(SExp::Cons(_, _, _))) => {
-                    argument_captures.insert(capture.clone(), bf.clone());
+                    argument_captures.insert(capture, bf.clone());
                     create_argument_captures(argument_captures, formed_arguments, substructure)
                 }
                 (None, BodyForm::Quoted(SExp::Cons(_, fa, ra))) => {
@@ -139,7 +139,7 @@ fn create_argument_captures(
                     )
                 }
                 (Some((capture, substructure)), bf) => {
-                    argument_captures.insert(capture.clone(), Rc::new(bf.clone()));
+                    argument_captures.insert(capture, Rc::new(bf.clone()));
                     create_argument_captures(argument_captures, formed_arguments, substructure)
                 }
                 (None, bf) => {
@@ -171,9 +171,8 @@ fn create_argument_captures(
             if let Some((capture, substructure)) = is_at_capture(f.clone(), r.clone()) {
                 let bfa = get_bodyform_from_arginput(l, af);
                 let bfb = get_bodyform_from_arginput(l, ar);
-                let fused_arguments =
-                    Rc::new(make_operator2(l, "c".to_string(), bfa.clone(), bfb.clone()));
-                argument_captures.insert(capture.clone(), fused_arguments);
+                let fused_arguments = Rc::new(make_operator2(l, "c".to_string(), bfa, bfb));
+                argument_captures.insert(capture, fused_arguments);
                 create_argument_captures(argument_captures, formed_arguments, substructure)
             } else {
                 create_argument_captures(argument_captures, af, f.clone())?;
@@ -195,8 +194,7 @@ fn create_argument_captures(
             function_arg_spec.loc(),
             format!(
                 "not yet supported argument alternative: ArgInput {:?} SExp {}",
-                formed_arguments,
-                function_arg_spec.to_string()
+                formed_arguments, function_arg_spec
             ),
         )),
     }
@@ -211,7 +209,7 @@ fn arg_inputs_primitive(arginputs: Rc<ArgInputs>) -> bool {
 
 fn build_argument_captures(
     l: &Srcloc,
-    arguments_to_convert: &Vec<Rc<BodyForm>>,
+    arguments_to_convert: &[Rc<BodyForm>],
     args: Rc<SExp>,
 ) -> Result<HashMap<Vec<u8>, Rc<BodyForm>>, CompileErr> {
     let mut formed_arguments = ArgInputs::Whole(Rc::new(BodyForm::Quoted(SExp::Nil(l.clone()))));
@@ -230,7 +228,7 @@ fn build_argument_captures(
 }
 
 fn make_prim_call(l: Srcloc, prim: Rc<SExp>, args: Rc<SExp>) -> Rc<SExp> {
-    Rc::new(SExp::Cons(l.clone(), prim, args))
+    Rc::new(SExp::Cons(l, prim, args))
 }
 
 pub fn build_reflex_captures(captures: &mut HashMap<Vec<u8>, Rc<BodyForm>>, args: Rc<SExp>) {
@@ -245,7 +243,7 @@ pub fn build_reflex_captures(captures: &mut HashMap<Vec<u8>, Rc<BodyForm>>, args
             if let Some((capture, substructure)) = is_at_capture(a.clone(), b.clone()) {
                 captures.insert(
                     capture.clone(),
-                    Rc::new(BodyForm::Value(SExp::Atom(l.clone(), capture.clone()))),
+                    Rc::new(BodyForm::Value(SExp::Atom(l.clone(), capture))),
                 );
                 build_reflex_captures(captures, substructure);
             } else {
@@ -264,7 +262,7 @@ pub fn dequote(l: Srcloc, exp: Rc<BodyForm>) -> Result<Rc<SExp>, CompileErr> {
             l,
             format!(
                 "not a quoted result in macro expansion: {} {:?}",
-                exp.to_sexp().to_string(),
+                exp.to_sexp(),
                 exp
             ),
         )),
@@ -289,7 +287,7 @@ pub fn first_of_alist(lst: Rc<SExp>) -> Result<Rc<SExp>, CompileErr> {
         SExp::Cons(_, f, _) => Ok(f.clone()),
         _ => Err(CompileErr(
             lst.loc(),
-            format!("No first element of {}", lst.to_string()),
+            format!("No first element of {}", lst),
         )),
     }
 }
@@ -299,7 +297,7 @@ pub fn second_of_alist(lst: Rc<SExp>) -> Result<Rc<SExp>, CompileErr> {
         SExp::Cons(_, _, r) => first_of_alist(r.clone()),
         _ => Err(CompileErr(
             lst.loc(),
-            format!("No second element of {}", lst.to_string()),
+            format!("No second element of {}", lst),
         )),
     }
 }
@@ -312,23 +310,17 @@ fn synthesize_args(
         SExp::Atom(_, name) => env.get(name).map(|x| Ok(x.clone())).unwrap_or_else(|| {
             Err(CompileErr(
                 template.loc(),
-                format!(
-                    "Argument {} referenced but not in env",
-                    template.to_string()
-                ),
+                format!("Argument {} referenced but not in env", template),
             ))
         }),
         SExp::Cons(l, f, r) => {
             if let Some((capture, _substructure)) = is_at_capture(f.clone(), r.clone()) {
-                synthesize_args(Rc::new(SExp::Atom(l.clone(), capture.clone())), env)
+                synthesize_args(Rc::new(SExp::Atom(l.clone(), capture)), env)
             } else {
                 Ok(Rc::new(BodyForm::Call(
                     l.clone(),
                     vec![
-                        Rc::new(BodyForm::Value(SExp::atom_from_string(
-                            template.loc(),
-                            &"c".to_string(),
-                        ))),
+                        Rc::new(BodyForm::Value(SExp::atom_from_string(template.loc(), "c"))),
                         synthesize_args(f.clone(), env)?,
                         synthesize_args(r.clone(), env)?,
                     ],
@@ -338,12 +330,12 @@ fn synthesize_args(
         SExp::Nil(l) => Ok(Rc::new(BodyForm::Quoted(SExp::Nil(l.clone())))),
         _ => Err(CompileErr(
             template.loc(),
-            format!("unknown argument template {}", template.to_string()),
+            format!("unknown argument template {}", template),
         )),
     }
 }
 
-fn reflex_capture(name: &Vec<u8>, capture: Rc<BodyForm>) -> bool {
+fn reflex_capture(name: &[u8], capture: Rc<BodyForm>) -> bool {
     match capture.borrow() {
         BodyForm::Value(SExp::Atom(_, n)) => n == name,
         _ => false,
@@ -462,23 +454,23 @@ fn promote_program_to_bodyform(
             }
 
             // Process tails to change bare numbers to (@ n)
-            let args = promote_args_to_bodyform(h.clone(), t.clone(), env.clone())?;
+            let args = promote_args_to_bodyform(h.clone(), t.clone(), env)?;
             Ok(Rc::new(BodyForm::Call(program.loc(), args)))
         }
         SExp::Integer(_, n) => {
             // A program that is an atom refers to a position
             // in the environment.
-            Ok(choose_from_env_by_path(n.clone(), env.clone()))
+            Ok(choose_from_env_by_path(n.clone(), env))
         }
         SExp::QuotedString(_, _, v) => {
             // Treated as integer path.
             let integer = number_from_u8(v);
-            Ok(choose_from_env_by_path(integer, env.clone()))
+            Ok(choose_from_env_by_path(integer, env))
         }
         SExp::Atom(_, v) => {
             // Treated as integer path.
             let integer = number_from_u8(v);
-            Ok(choose_from_env_by_path(integer, env.clone()))
+            Ok(choose_from_env_by_path(integer, env))
         }
         _ => {
             let borrowed_program: &SExp = program.borrow();
@@ -502,30 +494,6 @@ fn match_i_op(candidate: Rc<BodyForm>) -> Option<(Rc<BodyForm>, Rc<BodyForm>, Rc
     None
 }
 
-fn fake_body_of_env(l: Srcloc, env: &HashMap<Vec<u8>, Rc<BodyForm>>) -> Rc<BodyForm> {
-    let mut composed = Rc::new(BodyForm::Quoted(SExp::Nil(l.clone())));
-    let mut sorted_env: Vec<Vec<u8>> = env.iter().map(|nv| nv.0.clone()).collect();
-    sorted_env.sort();
-    for nv in sorted_env.iter() {
-        if let Some(v) = env.get(nv) {
-            let entry = Rc::new(BodyForm::Call(
-                v.loc(),
-                vec![
-                    Rc::new(BodyForm::Value(SExp::Atom(v.loc(), nv.clone()))),
-                    v.clone(),
-                ],
-            ));
-            composed = Rc::new(BodyForm::Call(l.clone(), vec![entry, composed]));
-        }
-    }
-    composed
-}
-
-fn compute_hash_of_apply(body: Rc<BodyForm>, env: Rc<BodyForm>) -> Vec<u8> {
-    let composed = Rc::new(BodyForm::Call(body.loc(), vec![body.clone(), env.clone()]));
-    sha256tree(composed.to_sexp())
-}
-
 fn flatten_expression_to_names_inner(collection: &mut HashSet<Vec<u8>>, expr: Rc<SExp>) {
     match expr.borrow() {
         SExp::Cons(_, a, b) => {
@@ -585,6 +553,7 @@ impl Evaluator {
         }
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn invoke_macro_expansion(
         &self,
         allocator: &mut Allocator,
@@ -593,7 +562,7 @@ impl Evaluator {
         call_loc: Srcloc,
         program: Rc<CompileForm>,
         prog_args: Rc<SExp>,
-        arguments_to_convert: &Vec<Rc<BodyForm>>,
+        arguments_to_convert: &[Rc<BodyForm>],
         env: &HashMap<Vec<u8>, Rc<BodyForm>>,
     ) -> Result<Rc<BodyForm>, CompileErr> {
         // Pass the SExp representation of the expressions into
@@ -605,17 +574,16 @@ impl Evaluator {
             macro_args = Rc::new(SExp::Cons(l.clone(), arg_repr, macro_args));
         }
 
-        let macro_expansion =
-            self.expand_macro(allocator, l.clone(), program.clone(), macro_args)?;
+        let macro_expansion = self.expand_macro(allocator, l.clone(), program, macro_args)?;
 
-        if let Ok(input) = dequote(call_loc.clone(), macro_expansion.clone()) {
+        if let Ok(input) = dequote(call_loc, macro_expansion.clone()) {
             let frontend_macro_input = Rc::new(SExp::Cons(
                 l.clone(),
-                Rc::new(SExp::atom_from_string(l.clone(), &"mod".to_string())),
+                Rc::new(SExp::atom_from_string(l.clone(), "mod")),
                 Rc::new(SExp::Cons(
                     l.clone(),
                     prog_args.clone(),
-                    Rc::new(SExp::Cons(l.clone(), input, Rc::new(SExp::Nil(l.clone())))),
+                    Rc::new(SExp::Cons(l.clone(), input, Rc::new(SExp::Nil(l)))),
                 )),
             ));
 
@@ -625,7 +593,7 @@ impl Evaluator {
                     visited,
                     prog_args.clone(),
                     env,
-                    program.exp.clone(),
+                    program.exp,
                     false,
                 )
             })
@@ -640,30 +608,31 @@ impl Evaluator {
         }
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn invoke_primitive(
         &self,
         allocator: &mut Allocator,
         visited: &mut HashMap<Vec<u8>, Rc<BodyForm>>,
         l: Srcloc,
-        call_name: &Vec<u8>,
-        parts: &Vec<Rc<BodyForm>>,
+        call_name: &[u8],
+        parts: &[Rc<BodyForm>],
         body: Rc<BodyForm>,
         prog_args: Rc<SExp>,
-        arguments_to_convert: &Vec<Rc<BodyForm>>,
+        arguments_to_convert: &[Rc<BodyForm>],
         env: &HashMap<Vec<u8>, Rc<BodyForm>>,
         only_inline: bool,
     ) -> Result<Rc<BodyForm>, CompileErr> {
         let mut all_primitive = true;
-        let mut target_vec: Vec<Rc<BodyForm>> = parts.clone();
+        let mut target_vec: Vec<Rc<BodyForm>> = parts.to_owned();
 
-        if call_name == &"@".as_bytes().to_vec() {
+        if call_name == "@".as_bytes() {
             // Synthesize the environment for this function
             Ok(Rc::new(BodyForm::Quoted(SExp::Cons(
                 l.clone(),
-                Rc::new(SExp::Nil(l.clone())),
+                Rc::new(SExp::Nil(l)),
                 prog_args,
             ))))
-        } else if call_name == &"com".as_bytes().to_vec() {
+        } else if call_name == "com".as_bytes() {
             let mut end_of_list = Rc::new(SExp::Cons(
                 l.clone(),
                 arguments_to_convert[0].to_sexp(),
@@ -677,7 +646,7 @@ impl Evaluator {
             let use_body = SExp::Cons(
                 l.clone(),
                 Rc::new(SExp::Atom(l.clone(), "mod".as_bytes().to_vec())),
-                Rc::new(SExp::Cons(l.clone(), prog_args.clone(), end_of_list)),
+                Rc::new(SExp::Cons(l, prog_args, end_of_list)),
             );
 
             let compiled = self.compile_code(allocator, false, Rc::new(use_body))?;
@@ -715,7 +684,7 @@ impl Evaluator {
                         match self.run_prim(
                             allocator,
                             l.clone(),
-                            make_prim_call(l.clone(), prim.clone(), Rc::new(converted_args)),
+                            make_prim_call(l.clone(), prim, Rc::new(converted_args)),
                             Rc::new(SExp::Nil(l.clone())),
                         ) {
                             Ok(res) => Ok(res),
@@ -740,7 +709,7 @@ impl Evaluator {
                         l.clone(),
                         format!(
                             "Don't yet support this call type {} {:?}",
-                            body.to_sexp().to_string(),
+                            body.to_sexp(),
                             body
                         ),
                     ))
@@ -790,7 +759,7 @@ impl Evaluator {
             }
 
             visited.insert(
-                where_from_vec.clone(),
+                where_from_vec,
                 Rc::new(BodyForm::Call(
                     maybe_condition.loc(),
                     vec![x_head.clone(), cond.clone()],
@@ -811,7 +780,7 @@ impl Evaluator {
                 visited,
                 Rc::new(BodyForm::Call(
                     iffalse.loc(),
-                    vec![apply_head, iffalse.clone(), env.clone()],
+                    vec![apply_head, iffalse.clone(), env],
                 )),
             );
 
@@ -860,18 +829,19 @@ impl Evaluator {
         Ok(body)
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn handle_invoke(
         &self,
         allocator: &mut Allocator,
         visited: &mut HashMap<Vec<u8>, Rc<BodyForm>>,
         l: Srcloc,
         call_loc: Srcloc,
-        call_name: &Vec<u8>,
+        call_name: &[u8],
         _head_expr: Rc<BodyForm>,
-        parts: &Vec<Rc<BodyForm>>,
+        parts: &[Rc<BodyForm>],
         body: Rc<BodyForm>,
         prog_args: Rc<SExp>,
-        arguments_to_convert: &Vec<Rc<BodyForm>>,
+        arguments_to_convert: &[Rc<BodyForm>],
         env: &HashMap<Vec<u8>, Rc<BodyForm>>,
         only_inline: bool,
     ) -> Result<Rc<BodyForm>, CompileErr> {
@@ -880,20 +850,20 @@ impl Evaluator {
             Some(HelperForm::Defmacro(l, _name, _args, program)) => self.invoke_macro_expansion(
                 allocator,
                 visited,
-                l.clone(),
-                call_loc.clone(),
+                l,
+                call_loc,
                 program,
-                prog_args.clone(),
+                prog_args,
                 arguments_to_convert,
                 env,
             ),
             Some(HelperForm::Defun(_, _, inline, args, fun_body)) => {
                 if !inline && only_inline {
-                    return Ok(body.clone());
+                    return Ok(body);
                 }
 
                 let argument_captures_untranslated =
-                    build_argument_captures(&call_loc, &arguments_to_convert, args.clone())?;
+                    build_argument_captures(&call_loc, arguments_to_convert, args.clone())?;
 
                 let mut argument_captures = HashMap::new();
                 // Do this to protect against misalignment
@@ -914,7 +884,7 @@ impl Evaluator {
                 self.shrink_bodyform_visited(
                     allocator,
                     visited,
-                    args.clone(),
+                    args,
                     &argument_captures,
                     fun_body,
                     only_inline,
@@ -924,7 +894,7 @@ impl Evaluator {
                 .invoke_primitive(
                     allocator,
                     visited,
-                    l.clone(),
+                    l,
                     call_name,
                     parts,
                     body,
@@ -953,33 +923,33 @@ impl Evaluator {
                 self.shrink_bodyform_visited(
                     allocator,
                     visited,
-                    prog_args.clone(),
+                    prog_args,
                     &updated_bindings,
                     body.clone(),
                     only_inline,
                 )
             }
             BodyForm::Let(l, LetFormKind::Sequential, bindings, body) => {
-                if bindings.len() == 0 {
+                if bindings.is_empty() {
                     self.shrink_bodyform_visited(
                         allocator,
                         visited,
-                        prog_args.clone(),
+                        prog_args,
                         env,
                         body.clone(),
                         only_inline,
                     )
                 } else {
                     let first_binding_as_list: Vec<Rc<Binding>> =
-                        bindings.iter().take(1).map(|x| x.clone()).collect();
+                        bindings.iter().take(1).cloned().collect();
                     let rest_of_bindings: Vec<Rc<Binding>> =
-                        bindings.iter().skip(1).map(|x| x.clone()).collect();
+                        bindings.iter().skip(1).cloned().collect();
 
                     let updated_bindings = update_parallel_bindings(env, &first_binding_as_list);
                     self.shrink_bodyform_visited(
                         allocator,
                         visited,
-                        prog_args.clone(),
+                        prog_args,
                         &updated_bindings,
                         Rc::new(BodyForm::Let(
                             l.clone(),
@@ -998,7 +968,7 @@ impl Evaluator {
                     self.shrink_bodyform_visited(
                         allocator,
                         visited,
-                        prog_args.clone(),
+                        prog_args,
                         env,
                         literal_args,
                         only_inline,
@@ -1027,7 +997,7 @@ impl Evaluator {
                                         visited,
                                         prog_args.clone(),
                                         env,
-                                        x.clone(),
+                                        x,
                                         only_inline,
                                     )
                                 })
@@ -1042,13 +1012,16 @@ impl Evaluator {
             }
             BodyForm::Value(v) => Ok(Rc::new(BodyForm::Quoted(v.clone()))),
             BodyForm::Call(l, parts) => {
-                if parts.len() == 0 {
-                    return Err(CompileErr(l.clone(), format!("Impossible empty call list")));
+                if parts.is_empty() {
+                    return Err(CompileErr(
+                        l.clone(),
+                        "Impossible empty call list".to_string(),
+                    ));
                 }
 
                 let head_expr = parts[0].clone();
                 let arguments_to_convert: Vec<Rc<BodyForm>> =
-                    parts.iter().skip(1).map(|x| x.clone()).collect();
+                    parts.iter().skip(1).cloned().collect();
 
                 match head_expr.borrow() {
                     BodyForm::Value(SExp::Atom(call_loc, call_name)) => self.handle_invoke(
@@ -1058,7 +1031,7 @@ impl Evaluator {
                         call_loc.clone(),
                         call_name,
                         head_expr.clone(),
-                        &parts,
+                        parts,
                         body.clone(),
                         prog_args,
                         &arguments_to_convert,
@@ -1072,7 +1045,7 @@ impl Evaluator {
                         call_loc.clone(),
                         &u8_from_number(call_int.clone()),
                         head_expr.clone(),
-                        &parts,
+                        parts,
                         body.clone(),
                         prog_args,
                         &arguments_to_convert,
@@ -1081,7 +1054,7 @@ impl Evaluator {
                     ),
                     _ => Err(CompileErr(
                         l.clone(),
-                        format!("Don't know how to call {}", head_expr.to_sexp().to_string()),
+                        format!("Don't know how to call {}", head_expr.to_sexp()),
                     )),
                 }
             }
@@ -1148,15 +1121,15 @@ impl Evaluator {
         ));
 
         let compiled = self.compile_code(allocator, false, use_body)?;
-        self.run_prim(allocator, call_loc.clone(), compiled, args)
+        self.run_prim(allocator, call_loc, compiled, args)
     }
 
-    fn lookup_prim(&self, l: Srcloc, name: &Vec<u8>) -> Option<Rc<SExp>> {
+    fn lookup_prim(&self, l: Srcloc, name: &[u8]) -> Option<Rc<SExp>> {
         match self.prims.get(name) {
             Some(p) => Some(p.clone()),
             None => {
                 if name.len() == 1 {
-                    Some(Rc::new(SExp::Atom(l, name.clone())))
+                    Some(Rc::new(SExp::Atom(l, name.to_owned())))
                 } else {
                     None
                 }
@@ -1179,10 +1152,8 @@ impl Evaluator {
             args,
         )
         .map_err(|e| match e {
-            RunFailure::RunExn(_, s) => {
-                CompileErr(call_loc.clone(), format!("exception: {}", s.to_string()))
-            }
-            RunFailure::RunErr(_, s) => CompileErr(call_loc.clone(), s.clone()),
+            RunFailure::RunExn(_, s) => CompileErr(call_loc.clone(), format!("exception: {}", s)),
+            RunFailure::RunErr(_, s) => CompileErr(call_loc.clone(), s),
         })
         .map(|res| {
             let res_borrowed: &SExp = res.borrow();
@@ -1225,7 +1196,7 @@ impl Evaluator {
         self.helpers.push(h.clone());
     }
 
-    fn get_constant(&self, name: &Vec<u8>) -> Option<Rc<BodyForm>> {
+    fn get_constant(&self, name: &[u8]) -> Option<Rc<BodyForm>> {
         for h in self.helpers.iter() {
             if let HelperForm::Defconstant(_, n, body) = h {
                 if n == name {
